# -*- coding: utf-8 -*-

import random
from math import exp
import datetime as dt


random.seed(0)

nb_flights = 2000
nb_crew    = 100

cities_list = [ ("Bombay"   ,"Inde" )     , ("New York" ,"USA" )      , ("Tokyo"   , "Japon" )     , ("Paris",   "France" )       , ("Francfort", "Allemagne" ), 
                ("Abu Dhabi","EAU" )      , ("Aberdeen" ,"Ecosse" )   , ("Londres" , "Angleterre" ), ("Glasgow", "Ecosse" )       , ("Pekin"    , "Chine" ), 
                ("Sidney"   ,"Australie" ), ("Melbourne","Australie" ), ("Conakry" , "Guinée" )    , ("Dakar", "Senegal" )        , ("Dublin"   , "Irlande" ),
                ("Tel Aviv" ,"Israel" )   , ("Le Caire" ,"Egypte" )   , ("Djibouti", "Djibouti")   , ("Buenos Aires", "Argentine"), ("Santiago" , "Chili" ) ]

compagnies_list = [ ("Qatar Airways", "Qatar")    , ("Air France", "France"),  ("Iberia"  , "Espagne"), 
                    ("Lufthansa"    , "Allemagne"), ("Ryan Air"  , "Irlande"), ("Easy Jet", "Irlande"),
                    ("Delta Airline", "USA") ]

motor_models = ["SNECMA-123", "SNECMA-246", "GE-247", "GE-656", "ROLLS ROYCE-345"]

plane_models = [("AIRBUS", "A380"), ("AIRBUS", "A350"), ("BOEING", "787"), ("BOEING", "747")] 

compagnies_plane_proba = [ [ 0.25, 0.25, 0.25, 0.25],
                           [ 0.35, 0.25, 0.15, 0.25],
                           [ 0.25, 0.25, 0.25, 0.25],
                           [ 0.10, 0.10, 0.40, 0.40],
                           [ 0.10, 0.20, 0.30, 0.40],
                           [ 0.40, 0.30, 0.20, 0.10],
                           [ 0.50, 0.50, 0.0 , 0.0 ] ]

plane_motor_proba = [ [ 0.20, 0.20, 0.20, 0.20, 0.20],
                      [ 0.30, 0.25, 0.20, 0.15, 0.10],
                      [ 0.10, 0.15, 0.20, 0.25, 0.30],
                      [ 0.30, 0.15, 0.10, 0.15, 0.30] ]

def get_random_index( proba_array ):
  assert( abs( sum( proba_array ) - 1.0 ) < 1e-10 )
  x = random.random()
  for index, it_proba in enumerate( proba_array ):
    x -= it_proba
    if x < 0:
      return index
  assert(False)

def get_random_immat( comp_country ):
  return comp_country[0] + "-" + "".join( [ str(unichr( random.randint( 65, 90 ))) for it in range(4) ] )

def get_random_hours():
  if random.randint(0, 10) == 10:
    return 4000 + random.randint(0,4000)
  return random.randint(0, 4000)

def generate_planes_and_motors():
  ## Id;Motor_model_id;Nb_hours
  ## Id;Immatriculation;Plane_model_id;Motor_id;Compagny_id;Nb_hours
  plane_list = []
  motor_list = []
  
  current_id = 0
  for it_c, it_comp in enumerate( compagnies_list ):
    for it in range(10):
      it_plane_model = get_random_index( compagnies_plane_proba[ it_c ] )
      it_motor_model = get_random_index( plane_motor_proba[ it_plane_model ] )

      plane_list.append( ( current_id, get_random_immat( it_comp[1] ), it_plane_model, current_id, it_c, get_random_hours() ) )
      motor_list.append( ( current_id, it_motor_model, get_random_hours() ) )

      current_id += 1

  return ( plane_list, motor_list )

( plane_list, motor_list ) = generate_planes_and_motors() 

def generate_crew_dict():
  crew_dict = { it_c : [] for it_c in range( len( compagnies_list ) ) }
  crew_list =[]
  for it in range( nb_crew ):
    new_crew = ( it, random.randint( 35, 65 ) )
    crew_dict[ random.randint( 0, len( compagnies_list ) - 1 ) ].append( new_crew )
    crew_list.append( new_crew )
  return (crew_dict, crew_list)

(crew_dict, crew_list) = generate_crew_dict()

def generate_flights_and_other_things():
  incidents_list = []
  flight_list = []
  crews_plane_habilitation_list = []
  
  wrong_crew_plane = []
  for it_f_id in range( nb_flights ):
    compagny_id = random.randint( 0, len( compagnies_list ) - 1 )
    wrong_crew = random.randint( 0, 200 ) == 200
    plane_id = 10*compagny_id + random.randint( 0, 9 )
    
    if wrong_crew:
      for it_crew in crew_dict[ compagny_id ]:
        if not ((it_crew[0], plane_id) in crews_plane_habilitation_list):
          crew_id = it_crew[0]
          wrong_crew_plane.append( (it_crew[0], plane_id) )
          break
      else:
        wrong_crew = False

    if not wrong_crew:
      while True:
        crew_id = crew_dict[ compagny_id ][ random.randint( 0, len( crew_dict[ compagny_id ] ) - 1) ][0]
        if not (crew_id, plane_id) in wrong_crew_plane:
          break
      crews_plane_habilitation_list.append( ( crew_id, plane_id ) )

    captain_age = crew_list[ crew_id ][1]
    motor_model_id = motor_list[ plane_id ][ 1 ] 
    plane_hour = plane_list[ plane_id ][ 5 ]
    motor_hour = motor_list[ plane_id ][ 2 ]

    if wrong_crew:
      incident = True
    else:
      incident_proba = 0.1 + 0.2 * ( motor_model_id == 2 ) - 0.1 * ( compagny_id == 3 ) + 0.1 * (captain_age > 50 ) + 0.5 * ( 1 / ( 1 + exp( 8 - max( motor_hour, plane_hour ) / 1000 ) ))

      incident = random.random() < incident_proba

    if incident:
      incidents_list.append( it_f_id )

    ## Id;Plane_id;Crew_id;Departure_id;Arrival_id;Date
    departure_id = random.randint( 0, len( cities_list ) - 1 )
    arrival_id = departure_id
    while arrival_id == departure_id:
      arrival_id = random.randint( 0, len( cities_list ) - 1 )

    flight_list.append( ( it_f_id, plane_id, crew_id, departure_id, arrival_id, dt.date(2013, 1, 1) + dt.timedelta( random.randint( 0, 364) ) ))

  return ( incidents_list, flight_list, crews_plane_habilitation_list )

( incidents_list, flight_list, crews_plane_habilitation_list ) = generate_flights_and_other_things()

def write_Incident( csv_file, table_name, column_names ):
  for it in incidents_list:
    csv_file.write( str( it ) + "\n" )
      
def write_Flights( csv_file, table_name, column_names ):
  for it_f in flight_list:
    csv_file.write( ";".join( map( repr, it_f ) ) + "\n" )

def write_Crews( csv_file, table_name, column_names ):
  ## Id;Captain_age
  for it_crew in crew_list:
    csv_file.write( ";".join( map( str, it_crew ) ) + "\n" )
      
def write_Crews_planes_habilitation( csv_file, table_name, column_names ):
  for it_c_f in crews_plane_habilitation_list:
    csv_file.write( ";".join( map( str, it_c_f) ) + "\n" )
      
def write_Planes( csv_file, table_name, column_names ):
  for it_p in plane_list:
    csv_file.write( ";".join( map( repr, it_p ) ) + "\n" ) 
  
def write_Motors( csv_file, table_name, column_names ):
  for it_m in motor_list:
    csv_file.write( ";".join( map( repr, it_m ) ) + "\n" )

def write_Plane_models( csv_file, table_name, column_names ):
  for index, it_plane_model in enumerate( plane_models ):
    csv_file.write( "{0};{1};{2}\n".format( index, it_plane_model[0], it_plane_model[1] ) )   

def write_Motor_models( csv_file, table_name, column_names ):
  for index, it_motor_model in enumerate( motor_models ):
    csv_file.write( "{0};{1};{2}\n".format( index, it_motor_model, it_motor_model.split("-")[0] ) )

def write_Compagnies( csv_file, table_name, column_names ):
  for it_index, it_compagny in enumerate( compagnies_list ):
    csv_file.write( "{0};{1};{2}\n".format( it_index, it_compagny[0], it_compagny[1] ) )

def write_Cities( csv_file, table_name, column_names ):
  for it_index, it_cities in enumerate( cities_list ):
    csv_file.write( "{0};{1};{2}\n".format( it_index, it_cities[0], it_cities[1] ) )

FILE_NAME = "data_model.txt"

def write_files():
  current_file = None

  with open( FILE_NAME, 'r') as data_model_file:
    for it_line in data_model_file:
      if it_line.strip() == "":
        if current_file:
          with open( current_file[0] + ".csv", 'w' ) as current_csv_file:
            current_csv_file.write( ";".join( current_file[1] ) + "\n" )
            globals()["write_" + current_file[0]]( current_csv_file, *current_file )
          current_file = None
        continue

      if it_line.startswith(" "):
        assert( current_file )
        current_file[1].append( it_line.strip().split(" ")[0] )
      else:
        assert( current_file == None)
        current_file = ( it_line.strip(), [] )

if __name__ == "__main__":
  write_files()


